==> kdeutils-2.0.tar.bz2: kdeutils-2.0/kab/look_edit_tabaddresses.cc <==
/* -*- C++ -*-
   This file implements the tab to edit the addresses.
 
   the KDE addressbook
   
   $ Author: Mirko Boehm $
   $ Copyright: (C) 1996-2000, Mirko Boehm $
   $ Contact: mirko@kde.org
         http://www.kde.org $
   $ License: GPL with the following explicit clarification:
         This code may be linked against any version of the Qt toolkit
         from Troll Tech, Norway. $

   $Revision: 1.17 $	 
*/

#include "look_edit_tabaddresses.h"
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpopupmenu.h>
#include <qlabel.h>
#include <qframe.h>
#include <qiconset.h>
#include <qstring.h>
#include <qtoolbutton.h>
#include <qtooltip.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kdialogbase.h>
#include <klocale.h>
#include <qconfigDB.h>
#include <kdebug.h>
#include <kapp.h>
#include "dialog_insinuation.h"

#define KEY_ADDRESSTYPES "AddressTypes"

TabAddresses::TabAddresses(KabAPI* api, QWidget *parent)
  : TabBasic(api, parent),
    currentItem(0),
    modified(false)
{  
  QStringList types;
  Section *section;
  KeyValueMap *keys;
  unsigned int count;
  // -----
  const int Col0=0, Col1=1, Col2=2;
  int row=0; // counter to step through the rows
  layout = new QGridLayout( this, 12, 3, 6, 2 );
  // layout->addRowSpacing( 0, 6 );
  // layout->addRowSpacing( 2, 6 );
  // layout->addRowSpacing( 4, 6 );
  layout->setColStretch( Col2, 0 );
  ++row; // leave first row (0) empty
  // ----- first row (buttons, combobox):
  QWidget *helper = new QWidget( this );
  layout->addMultiCellWidget( helper, row, row, Col0, Col2 );  
  newAddress = new QPushButton( i18n("Add"), helper  );
  newAddress->setAutoDefault( false );
  QToolTip::add(newAddress, i18n("Add another address"));
  delAddress=new QPushButton( i18n("Remove"), helper );
  delAddress->setAutoDefault( false );
  QToolTip::add(delAddress, i18n("Delete this address"));
  connect(newAddress, SIGNAL(clicked()), SLOT(addAddress()));
  connect(delAddress, SIGNAL(clicked()), SLOT(deleteAddress()));
  selector = new QComboBox(helper);
  connect(selector, SIGNAL(activated(int)), this, SLOT(highlighted(int)));
  QHBoxLayout *hlay = new QHBoxLayout( helper, 0, 2 );
  hlay->addWidget( newAddress );
  hlay->addWidget( delAddress );
  hlay->addWidget(selector);
  hlay->setStretchFactor(selector, 1000);
  // hlay->addStretch(1);
  ++row;
  // ----- headline label and lineedit:
  QLabel *l_headline=new QLabel(i18n("Headline:"), this);
  layout->addWidget(l_headline, row, Col0);
  headline=new QLineEdit(this);
  layout->addMultiCellWidget(headline, row, row, Col1, Col2);
  connect(headline, SIGNAL(textChanged(const QString&)),
	  SLOT(headlineChanged(const QString&)));
  ++row;
  // ----- separator frame:
  QFrame *frame=new QFrame(this);
  frame->setFrameStyle(QFrame::HLine|QFrame::Sunken);
  layout->addMultiCellWidget(frame, row, row, Col0, Col2);
  ++row;
  layout->addRowSpacing(row, 12);
  ++row;
  // ----- the organisation line:
  l_position = new QLabel(i18n("Assignment:"), this );
  helper = new QWidget( this );
  layout->addWidget( l_position, row, Col0 );
  layout->addMultiCellWidget( helper, row, row, Col1, Col2 );
  position = new QLineEdit( helper );
  connect( position, SIGNAL(textChanged(const QString&)),
	   this, SLOT(addressChanged(const QString &)) );
  orgdata = new QPushButton( i18n("Insinuation..."), helper  );
  orgdata->setAutoDefault( false );
  QToolTip::add(orgdata, i18n("Edit organisational attributes"));
  connect(orgdata, SIGNAL(clicked()), SLOT(insinuation()));
  hlay = new QHBoxLayout( helper, 0, 2 );
  hlay->addWidget( position );
  hlay->addWidget( orgdata );
  ++row;
  ++row;
  // ----- street:
  l_street = new QLabel(i18n("Street:"), this );
  street   = new QLineEdit( this );
  connect( street, SIGNAL(textChanged(const QString&)),
	   this, SLOT(addressChanged(const QString &)) );
  layout->addWidget( l_street, row, 0 );
  layout->addMultiCellWidget( street, row, row, Col1, Col2 );
  ++row;
  // ----- city/state:
  //       labels:
  l_city   = new QLabel( i18n("City"), this );
  l_state  = new QLabel( i18n("State"), this );
  l_zip    = new QLabel( i18n("Zipcode"), this );
  layout->addWidget( l_city, row, Col0, AlignBottom );
  layout->addWidget( l_state, row, Col1, AlignBottom );
  layout->addWidget( l_zip, row, Col2, AlignBottom );
  ++row;
  //       lineedits:
  city  = new QLineEdit(this);
  state = new QLineEdit(this);
  zip   = new QLineEdit(this);
  connect( city, SIGNAL(textChanged(const QString&)),
	   this, SLOT(addressChanged(const QString &)) );
  connect( state, SIGNAL(textChanged(const QString&)),
	   this, SLOT(addressChanged(const QString &)) );
  connect( zip, SIGNAL(textChanged(const QString&)),
	   this, SLOT(addressChanged(const QString &)) );
  layout->addWidget( city, row, Col0 );
  layout->addWidget( state, row, Col1 );
  layout->addWidget( zip, row, Col2 );
  ++row;
  // ----- country:
  l_country = new QLabel( i18n("Country:"), this );
  country   = new QLineEdit( this );
  connect( country, SIGNAL(textChanged(const QString&)),
	   SLOT(addressChanged(const QString &)) );
  layout->addWidget( l_country, row, Col0 );
  layout->addMultiCellWidget( country, row, row, Col1, Col2 );
  ++row;
  // ----- finish the layout:
  layout->setRowStretch(row, 1);
  // ----- create the "choices of address types" menu:
  popupAddresses=new QPopupMenu;
  section=api->addressbook()->configurationSection();
  if(section==0)
    {
      kdDebug() << "TabAddresses ctor: configuration section not available."
		<< endl;
    } else {
      keys=section->getKeys();
      if(keys->get(KEY_ADDRESSTYPES, types))
	{ // WORK_TO_DO: reload menu when configuration changed
	  for(count=0; count<types.count(); ++count)
	    {
	      popupAddresses->insertItem(*types.at(count), count);
	    }
	} else {
	  kdDebug() << "TabAddresses ctor: address types not configured."
		    << endl;
	}
    }
}






#if 0

TabAddresses::TabAddresses(QWidget *parent)
  : TabBasic(parent),
    currentItem(0),
    modified(false)
{
  const int NoOfCols=3;
  int style; // currently unused
  int row=0;
  QHBoxLayout *hboxbuttons;
  QHBoxLayout *hboxposition;
  // -----
  layout=new QGridLayout(this, 100, NoOfCols, 3, 2);
  if(layout==0)
    {
      KMessageBox::sorry
	(this, i18n("Out of memory."),
	 i18n("General failure."));
      ::exit(-1);
    }
  for(int count=0; count<NoOfCols; ++count)
    { // make sure all columns get the same width:
      layout->setColStretch(count, 1);
    }
  // ----- create and add the buttons:
  hboxbuttons=new QHBoxLayout(layout);
  hboxbuttons->setDirection(QBoxLayout::LeftToRight);
  newAddress=new QPushButton(this);
  newAddress->setPixmap(BarIcon("filenew"));
  QToolTip::add(newAddress, i18n("Add another address"));
  hboxbuttons->addWidget(newAddress, row, 1);
  delAddress=new QPushButton(this);
  delAddress->setPixmap(BarIcon("run"));
  QToolTip::add(delAddress, i18n("Delete this address"));
  hboxbuttons->addWidget(delAddress, row, 2);
  hboxbuttons->addStretch();
  layout->addMultiCell(hboxbuttons, row, row, 0, 2);
  ++row; // new row
  // ----- create and add the selector:
  selector=new QComboBox(/* true, */ this);
  connect(selector, SIGNAL(activated(int)), SLOT(highlighted(int)));
  layout->addMultiCellWidget(selector, row, row, 0, 2);
  ++row; // new row
  // ----- create and place the input fields according to country style selected:
  //       place the position fields:
  hboxposition=new QHBoxLayout(layout);
  l_position=new QLabel(i18n("Assignment:"), this);
  hboxposition->addWidget(l_position, 0, 0); // added to the layout directly
  position=new QLineEdit(this);
  connect(position, SIGNAL(textChanged(const QString&)),
	  SLOT(addressChanged(const QString &)));
  hboxposition->addWidget(position, 0, 1); // added to the hboxlayout
  orgdata=new QPushButton(this);
  orgdata->setPixmap(BarIcon("down"));
  hboxposition->addWidget(orgdata, 0, 2);
  QToolTip::add(orgdata, i18n("Edit organisational attributes"));
  /*
    hboxposition->setStretchFactor(orgdata, 1);
    hboxposition->setStretchFactor(l_position, 1);
    hboxposition->setStretchFactor(position, 10000);
  */
  layout->addMultiCell(hboxposition, row, row, 0, 2);
  ++row; // new row
  // ----- the address fields:
  switch(style)
    {
    default: // american style, city state zipcode
      l_street=new QLabel(i18n("Street:"), this);
      layout->addWidget(l_street, row, 0);
      street=new QLineEdit(this);
      layout->addMultiCellWidget(street, row, row, 1, 2);
      connect(street, SIGNAL(textChanged(const QString&)),
	      SLOT(addressChanged(const QString &)));
      ++row; // new row
      l_city=new QLabel(i18n("City"), this);
      layout->addWidget(l_city, row, 0, AlignBottom);
      l_state=new QLabel(i18n("State"), this);
      layout->addWidget(l_state, row, 1, AlignBottom);
      l_zip=new QLabel(i18n("Zipcode"), this);
      layout->addWidget(l_zip, row, 2, AlignBottom);
      city=new QLineEdit(this);
      ++row; // new row
      layout->addWidget(city, row, 0);
      connect(city, SIGNAL(textChanged(const QString&)),
	      SLOT(addressChanged(const QString &)));
      state=new QLineEdit(this);
      layout->addWidget(state, row, 1);
      connect(state, SIGNAL(textChanged(const QString&)),
	      SLOT(addressChanged(const QString &)));
      zip=new QLineEdit(this);
      layout->addWidget(zip, row, 2);
      connect(zip, SIGNAL(textChanged(const QString&)),
	      SLOT(addressChanged(const QString &)));
      ++row; // new row
      l_country=new QLabel(i18n("Country:"), this);
      layout->addWidget(l_country, row, 0);
      country=new QLineEdit(this);
      layout->addMultiCellWidget(country, row, row, 1, 2);
      connect(country, SIGNAL(textChanged(const QString&)),
	      SLOT(addressChanged(const QString &)));
    }
  connect(newAddress, SIGNAL(clicked()), SLOT(addAddress()));
  connect(delAddress, SIGNAL(clicked()), SLOT(deleteAddress()));
  connect(selector, SIGNAL(textChanged(const QString&)),
	  SLOT(headlineChanged(const QString&)));
}
#endif


void TabAddresses::storeContents(AddressBook::Entry& entry)
{
  if(modified) storeChanges();
  entry.addresses=addresses;
}

void TabAddresses::setContents(const AddressBook::Entry& entry)
{
  currentItem=0;
  addresses=entry.addresses;
  fillSelector();
  fillFields();
}

void TabAddresses::fillSelector()
{
  std::list<AddressBook::Entry::Address>::iterator pos;
  // -----
  selector->clear();
  for(pos=addresses.begin(); pos!=addresses.end(); ++pos)
    {
      selector->insertItem((*pos).headline);
    }
  headline->setFocus();
}

std::list<AddressBook::Entry::Address>::iterator TabAddresses::currentAddress()
{
  std::list<AddressBook::Entry::Address>::iterator pos;
  int index=currentItem;
  // ----- find the current address:
  if(addresses.size()==0)
    {
      return addresses.end();
    } else {
      if((unsigned int)currentItem>=addresses.size() || currentItem<0)
	{
	  return addresses.end();
	} else {
	  index=currentItem;
	  pos=addresses.begin();
	  advance(pos, index);
	  return pos;
	}
    }
}

void TabAddresses::fillFields()
{
  std::list<AddressBook::Entry::Address>::iterator pos;
  QLineEdit **lineedits[]= { &street, &zip, &city, &state, &country, &position, &headline };
  int Size=sizeof(lineedits)/sizeof(lineedits[0]);
  int count;
  // ----- find the current address:
  pos=currentAddress();
  // ----- fill the fields:
  if(pos==addresses.end())
    {
      for(count=0; count<Size; ++count)
	{
	  (*lineedits[count])->setEnabled(false);
	  (*lineedits[count])->setText("");
	}
    } else {
      QString *strings[]=
      {
	  &(*pos).address, &(*pos).zip, &(*pos).town,
	  &(*pos).state, &(*pos).country, &(*pos).position,
	  &(*pos).headline
      };
      for(count=0; count<Size; ++count)
	{
	  (*lineedits[count])->setEnabled(true);
	  (*lineedits[count])->setText(*strings[count]);
	}
    }
}

void TabAddresses::storeChanges()
{
  std::list<AddressBook::Entry::Address>::iterator pos;
  QLineEdit **lineedits[]= { &street, &zip, &city, &state, &country, &position, &headline };
  int Size=sizeof(lineedits)/sizeof(lineedits[0]);
  int count;
  // ----- find the current address:
  pos=currentAddress();
  // ----- store the fields:
  if(pos!=addresses.end())
    {
      QString *strings[]=
      {
	  &(*pos).address, &(*pos).zip, &(*pos).town,
	    &(*pos).state, &(*pos).country, &(*pos).position, &(*pos).headline
      };
      for(count=0; count<Size; ++count)
	{
	  *strings[count]=(*lineedits[count])->text();
	}
    }
}

void TabAddresses::addAddress()
{
  int index;
  AddressBook::Entry::Address address;
  // -----
  if(modified) storeChanges();
  if(popupAddresses->count()>0)
    {
      index=popupAddresses->exec
	(newAddress->mapToGlobal(QPoint(0, newAddress->height())));
      if(index>=0)
	{
	  address.headline=popupAddresses->text(index);
	} else {
	  return;
	}
    } else {
      KMessageBox::information
	(this,
	 i18n("<qt>Did you know that you may predefine types of addresses?<br>"
	      "See <b>Edit->Configure this file</b>.</qt>"),
	 i18n("A note"),
	 "kab_note_address_types");
    }
  addresses.push_back(address);
  fillSelector();
  selector->setCurrentItem(selector->count()-1);
  currentItem=selector->currentItem();
  fillFields();
  addressChanged();
}

void TabAddresses::deleteAddress()
{
  int index;
  std::list<AddressBook::Entry::Address>::iterator pos;  
  // -----
  if(addresses.size()==0)
    {
      kapp->beep();
      return;
    }
  if(KMessageBox::questionYesNo
     (this, i18n("Really delete this address?"),
      i18n("Question"))==KMessageBox::Yes)
    {
      index=selector->currentItem();
      if(index<0)
	{
	  kapp->beep();
	  return;
	}
      selector->removeItem(index);
      pos=addresses.begin();
      advance(pos, index);
      addresses.erase(pos);
      fillSelector();
      if(selector->count()>0)
	{
	  selector->setCurrentItem(QMAX(selector->count()-1, index));
	}
      fillFields();
    } else {
      kapp->beep();
    }
}

void TabAddresses::highlighted(int index)
{
  kdDebug() << "TabAddresses::highlighted: " << index << " (was " << currentItem << ")." << endl;
  if(modified)
    {
      storeChanges();
      modified=false;
    }
  currentItem=selector->currentItem();
  fillFields();
  headline->setText(selector->currentText());
  headline->setFocus();
  // modified=false; // after selection, changed has been signaled!
}

void TabAddresses::addressChanged()
{
  modified=true;
  emit(changed());
}

void TabAddresses::headlineChanged(const QString &text)
{
  modified=true;
  emit(changed());
  selector->changeItem(text, currentItem);
  selector->update();
}

void TabAddresses::addressChanged(const QString &)
{
  addressChanged();
}

void TabAddresses::insinuation()
{
  std::list<AddressBook::Entry::Address>::iterator pos;  
  KDialogBase dialog
    (this, "InsinuationDialog", true, i18n("Insinuation"),
     KDialogBase::Ok|KDialogBase::Cancel,
     KDialogBase::Ok, true);
  InsinuationDialog in(&dialog);
  pos=addresses.begin();
  advance(pos, currentItem);
  in.setFields(*pos);
  dialog.setMainWidget(&in);
  if(dialog.exec())
    {
      in.getFields(*pos);
      emit(changed());
    } else {
    }
}

	       
#include "look_edit_tabaddresses.moc"

==> kdeutils-2.1.1.tar.bz2: kdeutils-2.1.1/kab/look_edit_tabaddresses.cc <==
/* -*- C++ -*-
   This file implements the tab to edit the addresses.

   the KDE addressbook

   $ Author: Mirko Boehm $
   $ Copyright: (C) 1996-2000, Mirko Boehm $
   $ Contact: mirko@kde.org
         http://www.kde.org $
   $ License: GPL with the following explicit clarification:
         This code may be linked against any version of the Qt toolkit
         from Troll Tech, Norway. $

   $Revision: 1.20 $	
*/

#include "look_edit_tabaddresses.h"
#include "kab_deliverylabelwidget.h"
#include <qcombobox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qlineedit.h>
#include <qpopupmenu.h>
#include <qlabel.h>
#include <qframe.h>
#include <qiconset.h>
#include <qstring.h>
#include <qtoolbutton.h>
#include <qmultilineedit.h>
#include <qtooltip.h>
#include <kmessagebox.h>
#include <kiconloader.h>
#include <kdialogbase.h>
#include <klocale.h>
#include <qconfigDB.h>
#include <kdebug.h>
#include <kapp.h>
#include <knotifyclient.h>
#include "dialog_insinuation.h"

#define KEY_ADDRESSTYPES "AddressTypes"

TabAddresses::TabAddresses(KabAPI* api, QWidget *parent)
  : TabBasic(api, parent),
    currentItem(0),
    modified(false)
{
  QStringList types;
  Section *section;
  KeyValueMap *keys;
  QHBoxLayout *layout;
  unsigned int count;
  // -----
  layout=new QHBoxLayout(this);
  layout->setAutoAdd(true);
  base=new TabAddressesBase(this);
  connect(base->newAddress, SIGNAL(clicked()), SLOT(addAddress()));
  connect(base->delAddress, SIGNAL(clicked()), SLOT(deleteAddress()));
  connect(base->selector, SIGNAL(activated(int)), this, SLOT(highlighted(int)));
  connect(base->headline, SIGNAL(textChanged(const QString&)),
	  SLOT(headlineChanged(const QString&)));
  connect(base->position, SIGNAL(textChanged(const QString&)),
	   this, SLOT(addressChanged(const QString &)) );
  connect(base->orgdata, SIGNAL(clicked()), SLOT(insinuation()));
  connect(base->street, SIGNAL(textChanged(const QString&)),
	   this, SLOT(addressChanged(const QString &)) );
  connect(base->city, SIGNAL(textChanged(const QString&)),
	   this, SLOT(addressChanged(const QString &)) );
  connect(base->state, SIGNAL(textChanged(const QString&)),
	   this, SLOT(addressChanged(const QString &)) );
  connect(base->zip, SIGNAL(textChanged(const QString&)),
	   this, SLOT(addressChanged(const QString &)) );
  connect(base->country, SIGNAL(textChanged(const QString&)),
	  SLOT(addressChanged(const QString &)) );
  connect(base->deliverylabel, SIGNAL(clicked()), SLOT(deliveryLabel()));
  // ----- create the "choices of address types" menu:
  popupAddresses=new QPopupMenu;
  section=api->addressbook()->configurationSection();
  if(section==0)
    {
      kdDebug() << "TabAddresses ctor: configuration section not available."
		<< endl;
    } else {
      keys=section->getKeys();
      if(keys->get(KEY_ADDRESSTYPES, types))
	{ // WORK_TO_DO: reload menu when configuration changed
	  for(count=0; count<types.count(); ++count)
	    {
	      popupAddresses->insertItem(*types.at(count), count);
	    }
	} else {
	  kdDebug() << "TabAddresses ctor: address types not configured."
		    << endl;
	}
    }
}

void TabAddresses::storeContents(AddressBook::Entry& entry)
{
  if(modified) storeChanges();
  entry.addresses=addresses;
}

void TabAddresses::setContents(const AddressBook::Entry& entry)
{
  currentItem=0;
  addresses=entry.addresses;
  fillSelector();
  fillFields();
}

void TabAddresses::fillSelector()
{
  std::list<AddressBook::Entry::Address>::iterator pos;
  // -----
  base->selector->clear();
  for(pos=addresses.begin(); pos!=addresses.end(); ++pos)
    {
      base->selector->insertItem((*pos).headline);
    }
  base->headline->setFocus();
}

std::list<AddressBook::Entry::Address>::iterator TabAddresses::currentAddress()
{
  std::list<AddressBook::Entry::Address>::iterator pos;
  int index=currentItem;
  // ----- find the current address:
  if(addresses.size()==0)
    {
      return addresses.end();
    } else {
      if((unsigned int)currentItem>=addresses.size() || currentItem<0)
	{
	  return addresses.end();
	} else {
	  index=currentItem;
	  pos=addresses.begin();
	  advance(pos, index);
	  return pos;
	}
    }
}

void TabAddresses::fillFields()
{
  std::list<AddressBook::Entry::Address>::iterator pos;
  QLineEdit **lineedits[]= 
  {
    &(base->street), &(base->zip), &(base->city), &(base->state), 
    &(base->country), &(base->position), &(base->headline)
  };
  int Size=sizeof(lineedits)/sizeof(lineedits[0]);
  int count;
  // ----- find the current address:
  pos=currentAddress();
  // ----- fill the fields:
  if(pos==addresses.end())
    {
      for(count=0; count<Size; ++count)
	{
	  (*lineedits[count])->setEnabled(false);
	  (*lineedits[count])->setText("");
	}
      base->orgdata->setEnabled(false);
      base->deliverylabel->setEnabled(false);
    } else {
      QString *strings[]=
      {
	  &(*pos).address, &(*pos).zip, &(*pos).town,
	  &(*pos).state, &(*pos).country, &(*pos).position,
	  &(*pos).headline
      };
      for(count=0; count<Size; ++count)
	{
	  (*lineedits[count])->setEnabled(true);
	  (*lineedits[count])->setText(*strings[count]);
	}
      base->orgdata->setEnabled(true);
      base->deliverylabel->setEnabled(true);
    }
}

void TabAddresses::storeChanges()
{
  std::list<AddressBook::Entry::Address>::iterator pos;
  QLineEdit **lineedits[]= 
  {
    &(base->street), &(base->zip), &(base->city), &(base->state), 
    &(base->country), &(base->position), &(base->headline)
  };
  int Size=sizeof(lineedits)/sizeof(lineedits[0]);
  int count;
  // ----- find the current address:
  pos=currentAddress();
  // ----- store the fields:
  if(pos!=addresses.end())
    {
      QString *strings[]=
      {
	  &(*pos).address, &(*pos).zip, &(*pos).town,
	    &(*pos).state, &(*pos).country, &(*pos).position, &(*pos).headline
      };
      for(count=0; count<Size; ++count)
	{
	  *strings[count]=(*lineedits[count])->text();
	}
    }
}

void TabAddresses::addAddress()
{
  int index;
  AddressBook::Entry::Address address;
  // -----
  if(modified) storeChanges();
  if(popupAddresses->count()>0)
    {
      index=popupAddresses->exec
 	(base->newAddress->mapToGlobal
	 (QPoint(0, base->newAddress->height())));
      if(index>=0)
	{
	  address.headline=popupAddresses->text(index);
	} else {
	  return;
	}
    } else {
      KMessageBox::information
	(this,
	 i18n("<qt>Did you know that you may predefine types of addresses?<br>"
	      "See <b>Edit->Configure this file</b>.</qt>"),
	 i18n("A note"),
	 "kab_note_address_types");
    }
  addresses.push_back(address);
  fillSelector();
  base->selector->setCurrentItem(base->selector->count()-1);
  currentItem=base->selector->currentItem();
  fillFields();
  addressChanged();
}

void TabAddresses::deleteAddress()
{
  int index;
  std::list<AddressBook::Entry::Address>::iterator pos;
  // -----
  if(addresses.size()==0)
    {
      KNotifyClient::beep();
      return;
    }
  if(KMessageBox::questionYesNo
     (this, i18n("Really delete this address?"),
      i18n("Question"))==KMessageBox::Yes)
    {
      index=base->selector->currentItem();
      if(index<0)
	{
	  KNotifyClient::beep();
	  return;
	}
      base->selector->removeItem(index);
      pos=addresses.begin();
      advance(pos, index);
      addresses.erase(pos);
      fillSelector();
      if(base->selector->count()>0)
	{
	  base->selector->setCurrentItem(QMAX(base->selector->count()-1, index));
	}
      fillFields();
    } else {
      KNotifyClient::beep();
    }
}

void TabAddresses::highlighted(int index)
{
  kdDebug() << "TabAddresses::highlighted: " << index << " (was " << currentItem << ")." << endl;
  if(modified)
    {
      storeChanges();
      modified=false;
    }
  currentItem=base->selector->currentItem();
  fillFields();
  base->headline->setText(base->selector->currentText());
  base->headline->setFocus();
  // modified=false; // after selection, changed has been signaled!
}

void TabAddresses::addressChanged()
{
  modified=true;
  emit(changed());
}

void TabAddresses::headlineChanged(const QString &text)
{
  modified=true;
  emit(changed());
  base->selector->changeItem(text, currentItem);
  base->selector->update();
}

void TabAddresses::addressChanged(const QString &)
{
  addressChanged();
}

void TabAddresses::insinuation()
{
  std::list<AddressBook::Entry::Address>::iterator pos;
  KDialogBase dialog
    (this, "InsinuationDialog", true, i18n("Insinuation"),
     KDialogBase::Ok|KDialogBase::Cancel,
     KDialogBase::Ok, true);
  InsinuationDialog in(&dialog);
  pos=addresses.begin();
  advance(pos, currentItem);
  in.setFields(*pos);
  dialog.setMainWidget(&in);
  if(dialog.exec())
    {
      in.getFields(*pos);
      emit(changed());
    } else {
    }
}

void TabAddresses::deliveryLabel()
{
  std::list<AddressBook::Entry::Address>::iterator pos;
  KDialogBase dialog
    (this, "DeliveryLabelDialog", true, i18n("Delivery label"),
     KDialogBase::Ok|KDialogBase::Cancel,
     KDialogBase::Ok, true);
  DeliveryLabelWidget widget(&dialog);
  // ----
  pos=addresses.begin();
  advance(pos, currentItem);
  widget.mleDelivLabel->setText((*pos).deliveryLabel);
  dialog.setMainWidget(&widget);
  if(dialog.exec())
    {
      (*pos).deliveryLabel=widget.mleDelivLabel->text();;
      emit(changed());
    } else {
    }
}
	
#include "look_edit_tabaddresses.moc"
